@@ -452,9 +452,97 @@ class Insight_Definition:
                self.edit_data_entity(data_id, i, parameters_settings[i])
         return data_id
 
-    def edit_data(self, data_id: str, df: pd.DataFrame, index:bool=False):
+
+    def create_qc(self, dataset_slug="", fields:list=[], filters=None, sorted_by=None,  dataset_id="", limit=50, offset=0, pivot=[]):
+        """
+        Pivot columns MUST be contained in field list
+        """
+        if(fields==[]):
+            print("SELECT FIELDS")
+            return
+
+        if sorted_by is None:
+            sorted_by = []
+
+        if filters is None:
+            filters = []
+
+        if(dataset_id=="" and dataset_slug!=""):
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            url = self._base_url_askdata + '/smartbot/agents/agentslug/' + self._slug + '/datasetslug/' + dataset_slug
+
+            headers = {"Authorization": "Bearer" + " " + self._token}
+            response = s.get(url=url, headers=headers)
+            response.raise_for_status()
+            r = response.json()
+
+            datasetId = r["datasetId"]
+
+        elif(dataset_id!=""):
+            datasetId=dataset_id
+        else:
+            print("SPECIFY A DATASET ID OR SLUG")
+            return
+
+        body = {
+            "agentId": self.agent_id,
+            "datasetId": datasetId,
+            "fields": fields,
+            "sortedBy": sorted_by,
+            "filters": filters,
+            "limit": limit,
+            "offset": offset,
+            "pivot": pivot
+        }
+
+        url = self.smart_insight_url + "/composed_queries/sdk"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {"Authorization": "Bearer" + " " + self._token}
+        response = s.post(url=url, json=body, headers=headers)
+        response.raise_for_status()
+
+        qc_id = response.json()["id"]
+
+        return qc_id
+
+    def add_query_composer_to_data_component(self, data_id, dataset_slug="", fields:list=[], filters=None, sorted_by=None,  dataset_id="", limit=50, offset=0, pivot=[]):
+
+        qc_id = self.create_qc(dataset_slug, fields, filters, sorted_by, dataset_id, limit, offset, pivot)
+
+        body = {
+            "qcId": qc_id
+        }
+
+        url = self.smart_insight_url+"/definitions/"+self.definition_id+"/data_queries/"+data_id+"/data"
+        #logging.info("URL: {}".format(url))
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        r = s.put(url=url, data=body, headers=headers)
+        r.raise_for_status()
+        self.components = r.json()["components"]
+
+    def edit_data(self, data_id: str, df: pd.DataFrame, index:bool=False, qc_id=None):
 
         body = df.to_json(orient="table", index=index)
+        if(qc_id!=None):
+            body["qcId"] = qc_id
+            
         url = self.smart_insight_url+"/definitions/"+self.definition_id+"/data_queries/"+data_id+"/data"
         #logging.info("URL: {}".format(url))
         s = requests.Session()
